from __future__ import unicode_literals

import re

from django.db.models import Model
from django.template import Library, Node, TemplateSyntaxError, Variable, VariableDoesNotExist
from django.utils.translation import ugettext_lazy

from ..utils import get_model


QUOTED_STRING = re.compile(r'^["\'](?P<noquotes>.+)["\']$')

register = Library()


def handle_var(value, context):
    if not value:
        return None
    stringval = QUOTED_STRING.search(value)
    if stringval:
        return stringval.group('noquotes')
    else:
        try:
            return Variable(value).resolve(context)
        except VariableDoesNotExist:
            return value


def handle_perm(parser, token):
    parts = token.split_contents()
    tag = parts[0]
    num_parts = len(parts)
    if num_parts > 1 and parts[num_parts - 2] == 'as':
        context_var = parts.pop()
        parts.pop()  # 'as'
        num_parts -= 2
    else:
        context_var = None
    if num_parts > 3:
        raise TemplateSyntaxError(ugettext_lazy("Too many arguments for '%(tag)s' tag (%(num)s)") % {'tag': tag, 'num': num_parts})
    try:
        perm = parts[1]
    except IndexError:
        raise TemplateSyntaxError(ugettext_lazy("'%(tag)s' tag takes at least one parameter") % {'tag': tag})
    try:
        obj_or_model = parts[2]
    except IndexError:
        obj_or_model = None
    return tag, perm, obj_or_model, context_var


def get_permission(tag, perm, obj_or_model, context):
    perm = handle_var(perm, context)
    model = handle_var(obj_or_model, context)
    if model and not isinstance(model, Model):
        model = get_model(model, raise_exception=True)
    try:
        request = context['request']
    except IndexError:
        raise TemplateSyntaxError(ugettext_lazy("'%(tag)' tag requires request context") % {'tag': tag})
    try:
        user = request['user']
    except IndexError:
        raise TemplateSyntaxError(ugettext_lazy("'%(tag)' tag requires 'user' in request context") % {'tag': tag})
    if model:
        return user.has_perm(perm, model)
    return user.has_perm(perm)


@register.tag
def perm(parser, token):
    tag, perm, obj_or_model, context_var = handle_perm(parser, token)
    return PermNode(perm, obj_or_model, context_var)


@register.tag
def ifperm(parser, token):
    tag, perm, obj_or_model, context_var = handle_perm(parser, token)
    # States
    default_states = ['ifperm', 'else']
    end_tag = 'endifperm'
    # Place to store the states and their values
    states = {}
    # Iterate over our context and find tokens
    while token.contents != end_tag:
        current = token.contents
        states[current.split()[0]] = parser.parse(default_states + [end_tag])
        token = parser.next_token()
    # Return node
    return IfPermNode(states, perm, obj_or_model)


class PermNode(Node):
    def __init__(self, perm, obj_or_model, context_var):
        self.perm = perm
        self.obj_or_model = obj_or_model
        self.context_var = context_var

    def render(self, context):
        result = get_permission('perm', self.perm, self.obj_or_model, context)
        if self.context_var:
            context[self.context_var] = result
            return ''
        return result


class IfPermNode(Node):
    def __init__(self, states, perm, obj_or_model):
        self.states = states
        self.perm = perm
        self.obj_or_model = obj_or_model

    def render(self, context):
        result = get_permission('perm', self.perm, self.obj_or_model, context)
        if result:
            index = 'ifperm'
        else:
            index = 'else'
        try:
            return self.states[index].render(context)
        except KeyError:
            return ''



from __future__ import unicode_literals

from django.contrib.auth.models import User
from django.template import Template, Context
from django.utils.unittest import TestCase
from django.db import models

from .decorators import permissions_for
from .permissions import ModelPermissions
from .exceptions import PermAppException
from .utils import get_model
from .permissions import load_permissions


# Dummy patterns to satisfy Django
urlpatterns = ()


class Person(models.Model):
    first_name = models.CharField(max_length=30)
    last_name = models.CharField(max_length=30)

    def user_can_visit(self, user):
        return user.is_staff

    def __unicode__(self):
        return '{first_name} {last_name}'.format(first_name=self.first_name, last_name=self.last_name).strip()


@permissions_for(Person)
class PersonPermissions(ModelPermissions):
    def has_perm_visit(self):
        # Let's ask the Person object
        return self.obj.user_can_visit(self.user)

    def get_queryset_perm_gamma(self):
        if self.user.username == 'gamma' or self.user.is_superuser:
            return self.model.objects.all()
        return self.model.objects.none()


# Load permissions
load_permissions()


def render_template(template, **context_args):
    """
    Create a template that loads perm. Result is stripped of whitespace.
    """
    template = Template("{% load perm %}" + template)
    return template.render(Context(context_args)).strip()


class UtilsTest(TestCase):
    def test_get_model(self):
        with self.assertRaises(PermAppException):
            get_model('does.NotExist', raise_exception=True)
        self.assertEqual(None, get_model('does.NotExist', raise_exception=False))
        self.assertEqual(Person, get_model('perm.Person', raise_exception=False))


class PermissionsTest(TestCase):
    def setUp(self):
        self.person = Person.objects.create(first_name='alpha', last_name='centauri')
        self.superuser = User.objects.create(username='alpha', is_superuser=True, is_staff=False)
        self.staff_user = User.objects.create(username='beta', is_superuser=False, is_staff=True)
        self.normal_user = User.objects.create(username='gamma', is_superuser=False, is_staff=False)

    def test_permission_does_not_exist(self):
        perm = 'does_not_exist'
        # True for superuser
        self.assertEqual(True, self.superuser.has_perm(perm))
        self.assertEqual(True, self.superuser.has_perm(perm, Person))
        self.assertEqual(True, self.superuser.has_perm(perm, self.person))
        # False for non-superusers
        self.assertEqual(False, self.staff_user.has_perm(perm))
        self.assertEqual(False, self.staff_user.has_perm(perm, Person))
        self.assertEqual(False, self.staff_user.has_perm(perm, self.person))

    def test_permission_gamma(self):
        perm = 'gamma'
        # True for superuser
        self.assertEqual(True, self.superuser.has_perm(perm))
        self.assertEqual(True, self.superuser.has_perm(perm, Person))
        self.assertEqual(True, self.superuser.has_perm(perm, self.person))
        # False for our staff user, since he is not named gamma
        self.assertEqual(False, self.staff_user.has_perm(perm))
        self.assertEqual(False, self.staff_user.has_perm(perm, Person))
        self.assertEqual(False, self.staff_user.has_perm(perm, self.person))
        # True for our normal user, since he is not named gamma
        self.assertEqual(False, self.normal_user.has_perm(perm))
        self.assertEqual(False, self.normal_user.has_perm(perm, Person))
        self.assertEqual(True, self.normal_user.has_perm(perm, self.person))

    def test_template(self):
        def _test_template(user, perm):
            template1 = '{{% perm "{perm}" person %}}'.format(perm=perm)
            template2 = '{{% perm "{perm}" person as var %}}{{{{ var }}}}'.format(perm=perm)
            result1 = render_template(template1, request={'user': user}, person=self.person)
            result2 = render_template(template2, request={'user': user}, person=self.person)
            self.assertEqual(result1, result2)
            return result1
        self.assertEqual('alpha centauri', render_template('{{ person }}', request={'user': None}, person=self.person))
        self.assertEqual('True', _test_template(self.superuser, 'does_not_exist'))
        self.assertEqual('False', _test_template(self.staff_user, 'does_not_exist'))
        self.assertEqual('True', _test_template(self.superuser, 'gamma'))
        self.assertEqual('False', _test_template(self.staff_user, 'gamma'))
        self.assertEqual('True', _test_template(self.normal_user, 'gamma'))

    def tearDown(self):
        self.superuser.delete()
        self.staff_user.delete()
        self.normal_user.delete()
        self.person.delete()